--[[
	Animcis knyvtr, a szksges megszortsokat
	teljest fggvnyek progress barral val
	futtatshoz.

	Az exec_animated fggvny helyes mkdshez az animland
	fggvnyeknek bizonyos idkznknt egy
	coroutine.yield(percent) hvssal vissza kell adniuk
	vezrlst, ahol percent a fggvny aktulis kszltsge,
	0 s 1 kztt.

	Ezen tl a fggvnyek (els) visszatrsi
	rtke ktelezen egy 0 s 1 kzti szm kell legyen
	(ajnlott az 1, tekintve hogy ez lesz az utols kszltsgi
	rtk, amit megjelentnk).

	Az animland fggvnyek ilyen szint megszortsa a
	visszatrsi rtkek szempontjbl nem igazn szerencss,
	erre ksbb rdemes lenne jobb megoldst keresni.
]]

-- Ltrehozzuk az "anim knyvtrat".
anim = {}

-- Kirajzolja a progress bart adott szegmensszmmal, amit az
-- els coroutine hvsnl adhatunk meg. A percent egy
-- 0 s 1 kz es szm.
anim.draw_progressbar =
	function(percent, segments)
		-- Default argumentumrtk legegyszerbb megadsa.
		segments = segments or 50

		while true do
			-- Egy carriage return beszrsval ugrunk
			-- mindig a sor elejre, hogy fellrjuk az elz
			-- progress bart.
			io.write("\r");
			io.write("[");

			-- A kapott szzalknak megfelel szm szegmenst
			-- rajzolunk ki.
			local k = 0;
			-- A for ciklusnl harmadik argumentumknt adhatjuk
			-- meg a lpskzt, ami itt a szegmensszm reciproka.
			for i = 0, percent, 1 / segments do
				io.write("=");
				k = k + 1
			end

			-- A maradk szegmensek helyre space kerl.
			for i = k + 1, segments do
				io.write(" ");
			end

			io.write("]");
			-- A string formattal irathatunk ki pl. szmokat formzva
			-- (mint a C-s printf-nl). Itt elhagyjuk a szzalkos
			-- rtk tizedesjegyeit.
			io.write(" "..(string.format("%i",percent*100)).."%")

			-- Ha mg nincs ksz az animlt folyamat, visszaadjuk
			-- a vezrlst a coroutine hvjnak.
			-- Klnben j sorba lpnk (egsz eddig egy sorban
			-- dolgoztunk), s kiugrunk a ciklusbl.
			if percent < 1 then
				percent = coroutine.yield()
			else
				io.write("\n")
				break
			end
		end
	end

-- Progress bar kirajzolsa mellett futtatja a kapott fggvnyt.
-- Az els argumentum a progress bar frisstsi gyakorisga
-- msodpercben. Ezt kveti a fggvny, majd a fggvny argumentumai.
anim.exec_animated =
	function(upd_int, f, ...)
		-- A coroutine.wrap fggvny segtsgvel a fggvny
		-- coroutine-knt kezelhet verzijt kapjuk. A sttuszt
		-- nem krdezhetjk le, mint "sima" coroutine-oknl, de
		-- most nincs is r szksg.
		local func = coroutine.wrap(f)
		local animate = coroutine.wrap(anim.draw_progressbar)

		local perc_compl = 0
		-- Az os.cloc() hvs a program indtsa ta eltelt idt
		-- adja vissza msodpercekben, kt tizedes pontossggal.
		-- Ezredmsodperc pontossg rendszerid lekrdezsre
		-- nincs beptett lehetsg.
		local start_time = os.clock()

		-- Amg a fggvny nincs ksz, folyamatosan hvogatjuk,
		-- s a megadott idkznknt frisstjk a progress bart a
		-- fggvnyhvsbl visszakapott legutbbi rtkkel.
		while perc_compl < 1 do
			if os.clock() - start_time >= upd_int then
				animate(perc_compl)
				start_time = os.clock()
			end
			-- A wrappelt fggvny visszatrsi rtkei megegyeznek
			-- az eredetivel. A kompatibilis fggvnyeket gy rtuk
			-- meg, hogy az els visszatrsi rtkk mindig 1 lesz,
			-- gy rathatjuk ki a 100%-os kszltsget (lehet ennl
			-- jobb megolds).
			-- A ret vltozban troljuk a kvetkez visszatrsi rtket,
			-- ami a fggvny "rdemi" visszatrsi rtke (csak egy
			-- darab, az sszes elmentst mshogy kne megoldani).
			perc_compl, ret = func(...)
		end
		-- Kiratjuk a 100%-os progress bart is.
		animate(perc_compl)

		-- Visszaadjuk a fggvny visszatrsi rtkt, ha van.
		return ret
	end
